/**
* 只野テーブル拡張プラグイン
* オプション：
* sortObject {Array} ソート対象ヘッダを設定(1=有効 0=無効) 
*   デフォルトはソート無効 有効にする場合は左から順にヘッダ列毎に指定
*   例：ヘッダ列が5行あり2番目と3番目の項目を有効にする場合
*     	$('#rows').tadanotable({
*		    sortObject:[0,1,1]
*	    });
*       ソート対象項目の最終位置まで設定したら以降は省略可能
* pageDisplayedResults {Number} 明細表示件数
* pageNumber {Number} ページャー番号個数
* pageTotalText {Text} ページ情報（合計テキスト）
* pageCountText {Text} ページ情報（件数テキスト）
* pageUnitText {Text} ページ情報（ページ表示テキスト）
* pageBeforeText {Text} ページャー（前ページ移動テキスト）
* pageNextText {Text} ページ情報（次ページ移動テキスト）
* 
* スタイル：
* テーブルスタイルは独自に設定が可能です。
* 内部で設定しているタグは下記cssクラス名に対してスタイルの適用をすれば操作可能です。
* ページ情報：page-info
* ソート表示：page-order
* ページャーエリア：    page-area
* ページャー前へ：page-before
* ページャー次へ：page-next
* ページャー先頭：page-top
* ページャー終了：page-end
* ページャー番号：page-number
*
* ライセンス：
* Copyright (c) 2018 Syakaijin Tadano
* 
* 以下に定める条件に従い、本ソフトウェアおよび関連文書のファイル（以下「ソフトウェア」）の複製を取得する
* すべての人に対し、ソフトウェアを無制限に扱うことを無償で許可します。これには、ソフトウェアの複製を使用、
* 複写、変更、結合、掲載、頒布、サブライセンス、および/または販売する権利、およびソフトウェアを提供する
* 相手に同じことを許可する権利も無制限に含まれます。
* 
* 上記の著作権表示および本許諾表示を、ソフトウェアのすべての複製または重要な部分に記載するものとします。
* 
* ソフトウェアは「現状のまま」で、明示であるか暗黙であるかを問わず、何らの保証もなく提供されます。
* ここでいう保証とは、商品性、特定の目的への適合性、および権利非侵害についての保証も含みますが、
* それに限定されるものではありません。 作者または著作権者は、契約行為、不法行為、またはそれ以外であろうと、
* ソフトウェアに起因または関連し、あるいはソフトウェアの使用またはその他の扱いによって生じる一切の請求、
* 損害、その他の義務について何らの責任も負わないものとします。
* 
*/
(function($) {

    /**
    * テーブル拡張（件数、ソート、ページャー）機能を提供
    * @param {options}
    */
    $.fn.tadanoTable = function(options) {

        /**
        * オプション
        */
        var settings = $.extend({
            sortObject: [],
            pageDisplayedResults: 10,
            pageNumber: 5,
            pageTotalText: "合計件数：",
            pageCountText: "件",
            pageUnitText: "ページ",
            pageBeforeText: "前へ",
            pageNextText: "次へ",
        }, options );

        /**
        * テーブルオブジェクト
        */
        var $element = this;

        /**
        * ソート対象データリスト
        */
        var dataList = [];

        /**
        * 内部操作タグID
        */
        var TABLE_ID = $element.attr('id');
        var HEAD_SPAN_ID = TABLE_ID + '-span';
        var KEY_HEDDEN_ID = TABLE_ID + '-hedden';
        var PAGE_MAX_ID = TABLE_ID + '-page-max';
        var PAGE_CNT_ID = TABLE_ID + '-page-cnt';
        var PAGE_NEXT_ID = TABLE_ID + '-page-next';
        var PAGE_BACK_ID = TABLE_ID + '-page-back';
        var PAGE_AREA_ID = TABLE_ID + '-page-area';
        var PAGE_TOP_ID = TABLE_ID + '-page-top';
        var PAGE_END_ID = TABLE_ID + '-page-end';

        /**
        * ソート制御
        */
        var DATA_ORDER = 'order';
        var ORDER_NONE = '0';
        var ORDER_ASC = '1';
        var ORDER_DESC = '2';

        /**
        * ページャー制御変数
        */
        var pageIndex = 0;
        var pageCount = 0;
        var pageMax = 0;

        /**
        * 関数
        */

        /**
        * ページ情報設定
        */
        function setPageInfo(){
            var settext = '';
            settext += settings.pageTotalText;
            settext += pageMax;
            settext += settings.pageCountText;
            settext += ' ';
            settext += (pageIndex + 1) + '/' + pageCount;
            settext += settings.pageUnitText;
            $('#' + PAGE_MAX_ID).text(settext);
        };

        /**
        * ページ制御
        */
        function controlPage(){
            var show = (pageIndex * settings.pageDisplayedResults) + settings.pageDisplayedResults;
            $('tbody tr', $element).each(function() {
                $(this).css('display','none');
            });
            
            for(var i = (pageIndex * settings.pageDisplayedResults); i < show; i++) {
                if(i >= pageMax){
                    break;
                }
                $('tbody', $element).children('tr').eq(i).css('display','');
            }
            
            //ページ情報設定
            setPageInfo();

            //ページャー表示
            displayPager();

        }

        /**
        * ページャー表示
        */
        function displayPager(){

            var next = Math.floor(settings.pageNumber / 2);
            var back = next;
            next = (next * 2) != settings.pageNumber ? next + (settings.pageNumber - (next * 2)) : next;
            
            var endcnt = pageIndex + next;
            var startcnt = pageIndex - back;

            if(endcnt > pageCount)
            {
                endcnt = pageCount;
                startcnt = endcnt - settings.pageNumber;
            }
            
            if(startcnt < 0){
                startcnt = 0;
                endcnt = settings.pageNumber;
            }
            
            if(startcnt < 0){
                startcnt = 0;
            }
            
            $('[id^=' + PAGE_CNT_ID + ']').each(function() {
                $(this).css('display','none');
            });
            
            for(var i = startcnt; i < endcnt; i++) {
                $('#' + PAGE_CNT_ID + (i + 1)).css('display','');
            }

            if(startcnt <= 0){
                $('#' + PAGE_TOP_ID).css('display','none');
            }else{
                $('#' + PAGE_TOP_ID).css('display','');
            }

            if(endcnt <= pageCount - 1){
                $('#' + PAGE_END_ID).css('display','');
            }else{
                $('#' + PAGE_END_ID).css('display','none');
            }

        }

        /**
        * イベント処理
        */

        //ヘッダクリック処理
        $('th', $element).click(function() {
            var span = $('[id^=' + HEAD_SPAN_ID + ']', this);
            var col = $('th', $element).index(this);
            
            if(settings.sortObject.length <= col || settings.sortObject[col] === 0)
            {
                return false;
            }
            
            col++;

            if (span.data(DATA_ORDER) == ORDER_DESC) {
                span.html('▲');
                span.data(DATA_ORDER, ORDER_ASC);
                dataList.sort(function(a, b) { return a[col] < b[col] ? -1 : 1; });
            } else {
                span.html('▼');
                span.data(DATA_ORDER, ORDER_DESC);
                dataList.sort(function(a, b) { return a[col] > b[col] ? -1 : 1; });
            }

            $(this).siblings().find('[id^=' + HEAD_SPAN_ID + ']').html('');
            $(this).siblings().find('[id^=' + HEAD_SPAN_ID + ']').data(DATA_ORDER, ORDER_NONE);

            var sortlist = [];
            for(var i = 0; i < dataList.length; i++) {
                var $tr = $('tbody tr #' + KEY_HEDDEN_ID + dataList[i][0], $element).parent().parent();
                sortlist.push($tr);
            }
            
            for(var i = 0; i < sortlist.length; i++) {
                $('tbody',$element).append(sortlist[i]);
            }

            controlPage();
        });

        //ページャ前へクリック
        $(document).on('click','#' + PAGE_BACK_ID, function() {
            pageIndex--;
            if(pageIndex < 0)
            {
                pageIndex = 0;
            }

            //ページ制御
            controlPage();
            return false;
        });
        
        //ページャ次へクリック
        $(document).on('click','#' + PAGE_NEXT_ID, function() {
            pageIndex++;
            if(pageIndex >= pageCount)
            {
                pageIndex = pageCount - 1;
            }

            //ページ制御
            controlPage();
            return false;
        });
        
        //ページャ先頭クリック
        $(document).on('click','#' + PAGE_TOP_ID, function() {
            pageIndex = 0;

            //ページ制御
            controlPage();
            return false;
        });

        //ページャ終了クリック
        $(document).on('click','#' + PAGE_END_ID, function() {
            pageIndex = pageCount - 1;

            //ページ制御
            controlPage();
            return false;
        });

        //ページャー番号クリック
        $(document).on('click','[id^=' + PAGE_CNT_ID + ']', function() {
            pageIndex = $(this).text();
            pageIndex--;

            //ページ制御
            controlPage();
            return false;
        });

        /**
        * 初期処理
        */

        //クリック可能ヘッダ設定
        for(var i = 0; i < settings.sortObject.length; i++) {
            if(settings.sortObject[i] === 0){
                continue;
            }
            
            var $th = $('thead tr', $element).eq(0).children('th').eq(i);
            $th.css('cursor','pointer');
            var tagid = HEAD_SPAN_ID + i;
            var $tag = $('<span>', {
                id: tagid,
                text: ''
            });

            $tag.addClass('page-order');
            $th.append($tag);
        }

        //データリスト作成
        $('tbody tr', $element).each(function() {
        
            var datachild = [];
            datachild.push(pageMax);
            
            for(var i = 0; i < settings.sortObject.length; i++) {
                if(settings.sortObject[i] === 0){
                    datachild.push(0);
                    continue;
                }
                
                datachild.push($(this).children('td').eq(i).text());
            }
            
            var tagid = KEY_HEDDEN_ID + pageMax
            var $hedden = $('<input>', {
                type: 'hidden',
                id: tagid,
                text: pageMax
            });
            
            $(this).children('td').eq(0).append($hedden);
            
            dataList.push(datachild);
            pageMax++;
        });
        
        //ページ件数設定
        pageCount = Math.ceil(pageMax / settings.pageDisplayedResults);

        //総ページ設定
        var $info = $('<div>', {
            id: PAGE_MAX_ID
        });
        
        $info.addClass('page-info');
        //ページ情報設定
        $element.before($info);

        //ページャー設定
        if((pageIndex + 1) !== pageCount)
        {
            var $pagearea = $('<div>', {
                id: PAGE_AREA_ID
            });
            
            $pagearea.addClass('page-area');

            var $tag = $('<a>', {
                id: PAGE_BACK_ID,
                href:'#',
                text:settings.pageBeforeText
            });
            
            $tag.addClass('page-before');
            $pagearea.append($tag);

            var $tag = $('<a>', {
                id: PAGE_TOP_ID,
                href:'#',
                text:'…'
            });

            $tag.addClass('page-top');
            $pagearea.append($tag);

            for(var i = 0; i < pageCount; i++) {
                $tag = $('<a>', {
                    id: PAGE_CNT_ID + (i + 1),
                    href:'#',
                    text:(i + 1)
                });
                
                $tag.addClass('page-number');

                $pagearea.append($tag);
            }

            var $tag = $('<a>', {
                id: PAGE_END_ID,
                href:'#',
                text:'…'
            });

            $tag.addClass('page-end');
            $pagearea.append($tag);

            $tag = $('<a>', {
                id: PAGE_NEXT_ID,
                href:'#',
                text:settings.pageNextText
            });

            $tag.addClass('page-next');
            $pagearea.append($tag);

            $element.after($pagearea);

        }

        //ページ制御
        controlPage();

        return $element;
    }
}( jQuery ));